import { unescapeGlob } from 'src/utils/glob-pattern-helper';

const unescapeGlobTests = [
  { input: 'h?llo', output: 'h?llo' },
  { input: 'h\\?llo', output: 'h?llo' },
  { input: '\\!hello', output: '!hello' },
  { input: '\\*hello', output: '*hello' },
  { input: 'hello\\*', output: 'hello*' },
  { input: 'h\\(a|e\\)llo', output: 'h(a|e)llo' },
  { input: 'h\\[a-e\\]llo', output: 'h[a-e]llo' },
  { input: 'h\\[^a\\]llo', output: 'h[^a]llo' },
  { input: 'h\\[a-e\\]llo\\\\:foo', output: 'h[a-e]llo\\:foo' },
  { input: 'h\\{a,e\\}llo', output: 'h{a,e}llo' },
  { input: 'h\\{a,e}llo', output: 'h{a,e}llo' },
  { input: 'h\\[a-e\\]llo\\\\\\*', output: 'h[a-e]llo\\*' },
  { input: 'h\\?(a)llo', output: 'h?(a)llo' },
  { input: 'hello/\\!\\(a\\)llo', output: 'hello/!(a)llo' },
  { input: 'hello/\\+(a)llo', output: 'hello/+(a)llo' },
  { input: 'hello/\\@(a)llo', output: 'hello/@(a)llo' },
  { input: 'hello/\\*(a)llo', output: 'hello/*(a)llo' },
  { input: 'hello/\\?(a)llo', output: 'hello/?(a)llo' },
];

describe('unescapeGlob', () => {
  unescapeGlobTests.forEach((test) => {
    it(`should be output: ${test.output} for input: ${test.input} `, async () => {
      const result = unescapeGlob(test.input);

      expect(result).toEqual(test.output);
    });
  });
});
