import { rte } from '../../../helpers/constants';
import { acceptLicenseTermsAndAddDatabaseApi } from '../../../helpers/database';
import { BrowserPage, CliPage } from '../../../pageObjects';
import { commonUrl, ossStandaloneConfig } from '../../../helpers/conf';
import { deleteStandaloneDatabaseApi } from '../../../helpers/api/api-database';
import { Common } from '../../../helpers/common';

const browserPage = new BrowserPage();
const cliPage = new CliPage();
const common = new Common();

const jsonInvalidStructure = '"{\"test\": 123"';
const title = 'Value will be saved as Unicode';
const reason = 'as it is not valid in the selected format.';
let keyName = common.generateWord(10);

fixture `Warning for invalid formatter value`
    .meta({
        type: 'regression',
        rte: rte.standalone
    })
    .page(commonUrl)
    .beforeEach(async() => {
        await acceptLicenseTermsAndAddDatabaseApi(ossStandaloneConfig, ossStandaloneConfig.databaseName);
        // await cliPage.sendCommandInCli(jsonCommand);
    })
    .afterEach(async() => {
        // Clear keys and database
        await cliPage.sendCommandInCli(`del ${keyName}`);
        await deleteStandaloneDatabaseApi(ossStandaloneConfig);
    });
test('Verify that user can see warning message when editing value', async t => {
    // Open key details
    await browserPage.addStringKey(keyName, '{"test": 123}');
    await browserPage.selectFormatter('JSON');
    await browserPage.editStringKeyValue(jsonInvalidStructure);
    await t
        // Verify that user sees warning message when value in selected format is not correct
        .expect(browserPage.changeValueWarning.visible).ok('Warning is not displayed')
        // Verify that tooltip has text "Value will be saved as Unicode as it is not valid in the selected format."
        .expect(browserPage.changeValueWarning.find('h4').withExactText(title).visible).ok('Title is not correct')
        .expect(browserPage.changeValueWarning.find('div').withExactText(reason).visible).ok('Reason is not correct');
    await t.click(browserPage.saveButton);
    // Verify that when user click on save button, value is saved in Unicode format
    await t.expect(browserPage.stringValueAsJson.exists).notOk('Value is not converted to Unicode');
    // Verify that user doesn't see warning message if saving value is correct in selected format
    await browserPage.editStringKeyValue('{"test": 123}');
    await t
        .expect(browserPage.changeValueWarning.visible).notOk('Warning is not displayed')
        .expect(browserPage.stringValueAsJson.exists).ok('Value is not converted to JSON object');
});
test('Verify that user can remove invalid format value warning the message by clicking on ESC button', async t => {
    keyName = common.generateWord(10);
    const keyValue = 'a:1:{s:8:"glossary";a:2:{s:5:"title";s:7:"example";s:8:"GlossDiv";a:2:{s:5:"title";s:1:"S";s:9:"GlossList";a:1:{s:10:"GlossEntry";a:3:{s:2:"ID";s:4:"SGML";s:8:"GlossDef";a:2:{s:4:"para";s:8:"language";s:12:"GlossSeeAlso";a:1:{i:0;s:3:"XML";}}s:8:"GlossSee";s:6:"markup";}}}}}';
    await browserPage.addHashKey(keyName, '5000', 'PHP Serialized', keyValue);
    await browserPage.selectFormatter('PHP serialized');
    await browserPage.editHashKeyValue(jsonInvalidStructure);
    await t.expect(browserPage.changeValueWarning.visible).ok('Warning is not displayed');
    await t.pressKey('esc');
    await t.expect(browserPage.changeValueWarning.visible).notOk('Warning is still displayed');
});
